@@ -1,3 +1,92 @@
+#' Solve_3comp
+#' 
+#' This function solves for the amounts or concentrations of a chemical in
+#' different tissues as functions of time based on the dose and dosing
+#' frequency.  It uses a three compartment model with partition coefficients.
+#'  function does. ~~
+#' 
+#' Note that the model parameters have units of hours while the model output is
+#' in days.
+#' 
+#' Default of NULL for doses.per.day solves for a single dose.
+#' 
+#' The compartments used in this model are the gutlumen, gut, liver, and
+#' rest-of-body, with the plasma equivalent to the liver plasma.
+#' 
+#' Model Figure 
+#' \if{html}{\figure{3comp.png}{options: width="60\%" alt="Figure: Three
+#' Compartment Model Schematic"}} 
+#' \if{latex}{\figure{3comp.pdf}{options: width=12cm alt="Figure: Three Compartment
+#' Model Schematic"}}
+#' 
+#' When species is specified as rabbit, dog, or mouse, the function uses the
+#' appropriate physiological data(volumes and flows) but substitues human
+#' fraction unbound, partition coefficients, and intrinsic hepatic clearance.
+#' 
+#' 
+#' 
+#' @param chem.name Either the chemical name, CAS number, or the parameters
+#' must be specified.
+#' @param chem.cas Either the chemical name, CAS number, or the parameters must
+#' be specified.
+#' @param times Optional time sequence for specified number of days.  The
+#' dosing sequence begins at the beginning of times.
+#' @param parameters Chemical parameters from parameterize_3comp function,
+#' overrides chem.name and chem.cas.
+#' @param days Length of the simulation.
+#' @param tsteps The number time steps per hour.
+#' @param daily.dose Total daily dose, mg/kg BW.
+#' @param dose Amount of a single dose, mg/kg BW.  Overwrites daily.dose.
+#' @param doses.per.day Number of doses per day.
+#' @param initial.values Vector containing the initial concentrations or
+#' amounts of the chemical in specified tissues with units corresponding to
+#' output.units.  Defaults are zero.
+#' @param plots Plots all outputs if true.
+#' @param suppress.messages Whether or not the output message is suppressed.
+#' @param species Species desired (either "Rat", "Rabbit", "Dog", "Mouse", or
+#' default "Human").
+#' @param iv.dose Simulates a single i.v. dose if true.
+#' @param output.units Desired units (either "mg/L", "mg", "umol", or default
+#' "uM").
+#' @param method Method used by integrator (deSolve).
+#' @param rtol Argument passed to integrator (deSolve).
+#' @param atol Argument passed to integrator (deSolve).
+#' @param default.to.human Substitutes missing animal values with human values
+#' if true (hepatic intrinsic clearance or fraction of unbound plasma).
+#' @param recalc.blood2plasma Recalculates the ratio of the amount of chemical
+#' in the blood to plasma using the input parameters, calculated with
+#' hematocrit, Funbound.plasma, and Krbc2pu.
+#' @param recalc.clearance Recalculates the the hepatic clearance
+#' (Clmetabolism) with new million.cells.per.gliver parameter.
+#' @param dosing.matrix Vector of dosing times or a matrix consisting of two
+#' columns or rows named "dose" and "time" containing the time and amount, in
+#' mg/kg BW, of each dose.
+#' @param adjusted.Funbound.plasma Uses adjusted Funbound.plasma when set to
+#' TRUE along with partition coefficients calculated with this value.
+#' @param regression Whether or not to use the regressions in calculating
+#' partition coefficients.
+#' @param restrictive.clearance Protein binding not taken into account (set to
+#' 1) in liver clearance if FALSE.
+#' @param minimum.Funbound.plasma Monte Carlo draws less than this value are set 
+#' equal to this value (default is 0.0001 -- half the lowest measured Fup in our
+#' dataset).
+#' @param ... Additional arguments passed to the integrator.
+#' @return A matrix of class deSolve with a column for time(in days) and each
+#' compartment, the plasma concentration, area under the curve, and a row for
+#' each time point.
+#' @author John Wambaugh and Robert Pearce
+#' @references Pearce, Robert G., et al. "Httk: R package for high-throughput
+#' toxicokinetics." Journal of statistical software 79.4 (2017): 1.
+#' @keywords Solve
+#' @examples
+#' 
+#' solve_3comp(chem.name='Bisphenol-A',doses.per.day=2,dose=.5,days=1,tsteps=2)
+#' params <-parameterize_3comp(chem.cas="80-05-7")
+#' solve_3comp(parameters=params)
+#' 
+#' @import deSolve 
+#' @export solve_3comp
+#' @useDynLib httk
 solve_3comp <- function(chem.name = NULL,
                     chem.cas = NULL,
                     times=NULL,
@@ -21,14 +110,24 @@ solve_3comp <- function(chem.name = NULL,
                     adjusted.Funbound.plasma=T,
                     regression=T,
                     restrictive.clearance = T,
+                    minimum.Funbound.plasma=0.0001,
                     ...)
 {
   Agutlumen <- Agut <- Aliver <- Arest <- Cgut <- Cliver <- Crest <- NULL
-  if(is.null(chem.cas) & is.null(chem.name) & is.null(parameters)) stop('Parameters, chem.name, or chem.cas must be specified.')
-  if (is.null(parameters)){
-    parameters <- parameterize_3comp(chem.cas=chem.cas,chem.name=chem.name,species=species,default.to.human=default.to.human,suppress.messages=suppress.messages,
-                                     adjusted.Funbound.plasma=adjusted.Funbound.plasma,regression=regression)
-  }else{
+  if (is.null(chem.cas) & is.null(chem.name) & is.null(parameters)) 
+    stop('Parameters, chem.name, or chem.cas must be specified.')
+  if (is.null(parameters))
+  {
+    parameters <- parameterize_3comp(
+                    chem.cas=chem.cas,
+                    chem.name=chem.name,
+                    species=species,
+                    default.to.human=default.to.human,
+                    suppress.messages=suppress.messages,
+                    adjusted.Funbound.plasma=adjusted.Funbound.plasma,
+                    regression=regression,
+                    minimum.Funbound.plasma=minimum.Funbound.plasma)
+  } else {
     if(!all(param.names.3comp %in% names(parameters)))stop(paste("Missing parameters:",paste(param.names.3comp[which(!param.names.3comp %in% names(parameters))],collapse=', '),".  Use parameters from parameterize_3comp."))
     if(any(param.names.pbtk[which(!param.names.pbtk %in% param.names.3comp)] %in% names(parameters)))stop("Parameters are from parameterize_pbtk.  Use parameters from parameterize_3comp.")
   }  
@@ -133,7 +232,7 @@ solve_3comp <- function(chem.name = NULL,
   }
   if(!restrictive.clearance) parameters$Clmetabolismc <- parameters$Clmetabolismc / parameters$Funbound.plasma
     
-  parameters[['CLmetabolismc']] <- parameters[['Clmetabolismc']] 
+  parameters[['Clmetabolismc']] <- parameters[['Clmetabolismc']] 
   parameters[['Fraction_unbound_plasma']] <- parameters[['Funbound.plasma']]
   parameters[['Ratioblood2plasma']] <- parameters[['Rblood2plasma']]
   names(parameters)[substr(names(parameters),1,1) == 'K'] <- gsub('2pu','2plasma',names(parameters)[substr(names(parameters),1,1) == 'K'])
@@ -166,7 +265,7 @@ solve_3comp <- function(chem.name = NULL,
   
   if(plots==T)
   {
-    plot(out,select=c(CompartmentsToInitialize,"Cplasma","AUC","Ametabolized","Atubules"))
+    graphics::plot(out,select=c(CompartmentsToInitialize,"Cplasma","AUC","Ametabolized","Atubules"))
   }
   
   out <- out[,c("time",CompartmentsToInitialize,"Cplasma","AUC","Ametabolized","Atubules")]
